==> cobertura-diff-coverage-1.0.2.tgz: package/cobertura-parse.js <==
const fs = require("fs").promises;
const xml2js = require("xml2js");
const util = require("util");
const parseString = util.promisify(xml2js.parseString);

/**
 * generate the report for the given file
 *
 * @param path: string
 * @param options: object
 * @return {Promise<{total: number, line: number, files: T[], branch: number}>}
 */
async function readCoverageFromXml(xmlString, options) {
  const { coverage } = await parseString(xmlString, {
    explicitArray: false,
    mergeAttrs: true,
  });
  const { packages } = coverage;
  const classes = processPackages(packages);
  const files = classes
    .filter(Boolean)
    .map((klass) => {
      return {
        ...calculateRates(klass),
        filename: klass["filename"],
        name: klass["name"],
        missing: missingLines(klass),
      };
    })
    .filter((file) => options.skipCovered === false || file.total < 100);
  return {
    ...calculateRates(coverage),
    files,
  };
}


function processPackages(packages) {
  if (packages.package instanceof Array) {
    return packages.package.map((p) => processPackage(p)).flat();
  } else if (packages.package) {
    return processPackage(packages.package);
  } else {
    return processPackage(packages);
  }
}

function processPackage(packageObj) {
  if (packageObj.classes && packageObj.classes.class instanceof Array) {
    return packageObj.classes.class;
  } else if (packageObj.classes && packageObj.classes.class) {
    return [packageObj.classes.class];
  } else if (packageObj.class && packageObj.class instanceof Array) {
    return packageObj.class;
  } else {
    return [packageObj.class];
  }
}

/**
 * returns coverage rates
 *
 * @param element: object
 * @returns {{total: number, line: number, branch: number}}
 */
function calculateRates(element) {
  const line = parseFloat(element["line-rate"]) * 100;
  const branch = parseFloat(element["branch-rate"]) * 100;
  const total = line && branch ? (line + branch) / 2 : line;
  return {
    total,
    line,
    branch,
  };
}

function getLines(klass) {
  if (klass.lines && klass.lines.line instanceof Array) {
    return klass.lines.line;
  } else if (klass.lines && klass.lines.line) {
    return [klass.lines.line];
  } else {
    return [];
  }
}

function missingLines(klass) {
  // Bail if line-rate says fully covered
  if (parseFloat(klass["line-rate"]) >= 1.0) return "";

  const lines = getLines(klass).sort(
    (a, b) => parseInt(a.number) - parseInt(b.number)
  );
  const statements = lines.map((line) => line.number);
  const misses = lines
    .filter((line) => parseInt(line.hits) < 1)
    .map((line) => line.number);
  return partitionLines(statements, misses);
}

function partitionLines(statements, lines) {
  /*
   * Detect sequences, with gaps according to 'statements',
   * in 'lines' and compress them in to a range format.
   *
   * Example:
   *
   * statements = [1,2,3,4,5,10,11,12,13,14,15,16]
   * lines =      [1,2,    5,10,11,   13,14,  ,16]
   * Returns: [[1, 2], [5, 11], [13, 14], [16, 16]]
   */
  const ranges = [];
  let start = null;
  let linesCursor = 0;
  let end;
  for (const statement of statements) {
    if (linesCursor >= lines.length) break;

    if (statement === lines[linesCursor]) {
      // (Consecutive) element from 'statements' matches
      // element from 'lines' at 'linesCursor'
      linesCursor += 1;
      if (start === null) start = statement;
      end = statement;
    } else if (start !== null) {
      // Consecutive elements are broken, an element from
      // 'statements' is missing from 'lines'
      ranges.push([start, end]);
      start = null;
    }
  }
  // (Eventually) close range running last iteration
  if (start !== null) ranges.push([start, end]);

  return ranges;
}


module.exports = {
    readCoverageFromXml
};

==> cobertura-diff-coverage-1.0.9.tgz: package/cobertura-parse.js <==
const fs = require("fs").promises;
const xml2js = require("xml2js");
const util = require("util");
const parseString = util.promisify(xml2js.parseString);

/**
 * generate the report for the given file
 *
 * @param path: string
 * @param options: object
 * @return {Promise<{total: number, line: number, files: T[], branch: number}>}
 */
async function readCoverageFromXml(xmlString, options) {
  const { coverage } = await parseString(xmlString, {
    explicitArray: false,
    mergeAttrs: true,
  });
  const { packages } = coverage;
  const classes = processPackages(packages);
  const files = classes
    .filter(Boolean)
    .map((klass) => {
      return {
        ...calculateRates(klass),
        file: klass["filename"],
        name: klass["name"],
        lines: missingLines(klass),
      };
    });
  return {
    ...calculateRates(coverage),
    files,
  };
}


function processPackages(packages) {
  if (packages.package instanceof Array) {
    return packages.package.map((p) => processPackage(p)).flat();
  } else if (packages.package) {
    return processPackage(packages.package);
  } else {
    return processPackage(packages);
  }
}

function processPackage(packageObj) {
  if (packageObj.classes && packageObj.classes.class instanceof Array) {
    return packageObj.classes.class;
  } else if (packageObj.classes && packageObj.classes.class) {
    return [packageObj.classes.class];
  } else if (packageObj.class && packageObj.class instanceof Array) {
    return packageObj.class;
  } else {
    return [packageObj.class];
  }
}

/**
 * returns coverage rates
 *
 * @param element: object
 * @returns {{total: number, line: number, branch: number}}
 */
function calculateRates(element) {
  const line = parseFloat(element["line-rate"]) * 100;
  const branch = parseFloat(element["branch-rate"]) * 100;
  const total = line && branch ? (line + branch) / 2 : line;
  return {
    total,
    line,
    branch,
  };
}

function getLines(klass) {
  if (klass.lines && klass.lines.line instanceof Array) {
    return klass.lines.line;
  } else if (klass.lines && klass.lines.line) {
    return [klass.lines.line];
  } else {
    return [];
  }
}

function missingLines(klass) {
  const lines = getLines(klass).sort(
    (a, b) => parseInt(a.number) - parseInt(b.number)
  );
  return partitionLines(lines);
}

function partitionLines(lines) {

  let finalLines = [];
  let linesCursor = 0;
  for (const statement of lines) {
    if(statement.branch === 'true')
    {
      const branchRate = statement['condition-coverage']
      const splitIndex = branchRate.indexOf('/')
      statement.branchesToCover = parseInt(branchRate.substring(splitIndex+1, splitIndex+2))
      statement.branchesCovered = parseInt(branchRate.substring(splitIndex-1, splitIndex))
      statement.number = parseInt(statement.number)
      statement.hits = parseInt(statement.hits)
    }
    finalLines.push(statement)
  }
  return finalLines;
}


module.exports = {
    readCoverageFromXml
};
